/*
 * Copyright (C) 2016, Ronny Fuchs
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import QtQuick.Window 2.0
import Ubuntu.Components.ListItems 1.3 as ListItem
import Diabetes_Diary_App 1.0

import "../Components"

Page{
    id: favoriteMealPage
    title: (state === "default") ? "Favorite Food" : "Delete Food"

    state: "default"
    states: [
        PageHeadState {
            name: "default"
            head: favoriteMealPage.head
            actions: [
                Action {
                    iconName: "search"
                    onTriggered: favoriteMealPage.state = "search"
                },
                Action {
                    iconName: "add"
                    onTriggered: carbsStore.createEntry();
                }
            ]
        },
        PageHeadState {
            name: "delete"
            head: favoriteMealPage.head
            backAction: Action {
                text: "back"
                iconName: "back"
                onTriggered: {
                    favoriteMealPage.state = "default"
                    resetHighlightedItem();
                }
            }
            actions: Action {
                iconName: "delete"
                onTriggered: {
                    carbsStore.removeEntry(favoriteMealPage.selectedId);
                    favoriteMealPage.state = "default"
                    resetHighlightedItem();
                }
            }
        },
        PageHeadState {
            name: "search"
            head: favoriteMealPage.head
            backAction: Action {
                text: "back"
                iconName: "back"
                onTriggered: {
                    searchText.text = "";
                    favoriteMealPage.state = "default"
                }
            }
            contents: TextField {
                id: searchText
                placeholderText: "Search food"
                inputMethodHints: Qt.ImhNoPredictiveText
                primaryItem: Icon {
                    height: parent.height - units.gu(2)
                    width: height
                    name: "search"
                }
                width: parent ? (parent.width - units.gu(2)) : undefined
                onTextChanged: carbsStoreSorted.search(text);
                onVisibleChanged: forceActiveFocus()
            }
        }
    ]

    property int selectedId: -1
    property var selectedItem: null

    function resetHighlightedItem(){
        selectedItem.highlighted = false
        carbsList.currentIndex = -1;
        selectedId = -1;
    }

    SortFilterModel{
        id: carbsStoreSorted
        function search(pattern) {
            filter.pattern = new RegExp(pattern,'i')
        }
        filter.property: "name"
        model: CarbsStore{
            id: carbsStore
        }
        sort.property: "name"
        sort.order: Qt.AscendingOrder
        sortCaseSensitivity: Qt.CaseInsensitive
    }

    ListView {
        id: carbsList
        anchors {left: parent.left; right: parent.right;}
        height: parent.height - y
        maximumFlickVelocity: height * 4
        flickDeceleration: height
        cacheBuffer: 3000
        model: carbsStoreSorted
        clip: true

        property int expandedIndex: -1
        property var expandedItem: null

        delegate: MealItem{
            id: mealItem
            anchors {
                left: parent.left;
                right: parent.right;
                margins: -units.gu(2)
            }
            entry: model
            expanded: model.expanded
            onSave: carbsStore.saveEntry(model.id,newName, newWeight, newCarbs);
            onPressAndHold: {
                mealItem.highlighted = true;
                carbsList.currentIndex = index;
                favoriteMealPage.selectedId = model.id;
                favoriteMealPage.selectedItem = carbsList.currentItem;
                favoriteMealPage.state = "delete";
            }
            onClicked: {
                if (carbsList.isInSelectionModel) {
                    carbsList.selectItem(index)
                }
                if(carbsList.expandedItem){
                    if (carbsList.expandedIndex !== index){
                        carbsList.expandedItem.expanded = false
                    }
                }
                if (expanded){
                    carbsList.currentIndex = index;
                    carbsList.expandedIndex = index;
                    carbsList.expandedItem = carbsList.currentItem;
                }
                else{
                    carbsList.currentIndex = -1;
                    carbsList.expandedIndex = -1;
                    carbsList.expandedItem = null;
                }
            }
        }
        section.property: "name"
        section.criteria: ViewSection.FirstCharacter
        section.delegate: ListItem.Empty{
            anchors {left: parent.left; right: parent.right}
            height: units.gu(4)
            highlightWhenPressed: false
            Label {
                anchors.fill: parent
                anchors.leftMargin: units.gu(2)
                text: section
                font.bold: true
                verticalAlignment: Text.AlignVCenter
                height: units.gu(3)
            }
        }
        Scrollbar {
            flickableItem: carbsList
            parent: carbsList.parent
        }
    }
    EmptyState{
        iconName: "dialog-warning-symbolic"
        title: i18n.tr("No entries!")
        subTitle: i18n.tr("Click on the + symbol to add an entry!")
        anchors.centerIn: parent
        visible: (carbsStore.count === 0) ? true : false
    }
}
