/*
 * Copyright (C) 2016, Ronny Fuchs
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.ListItems 1.3 as ListItem
import Ubuntu.Components.Themes.Ambiance 1.1
import "../Components"

ListItem.Expandable {
    id: mealExpand
    expandedHeight: mealCol.height

    property var entry: null
    property alias expanded: mealExpand.expanded
    property alias highlighted: itemTitle.selected

    signal save(var newName, var newWeight, var newCarbs)
    signal pressAndHold();
    signal clicked();

    Column{
        id: mealCol
        anchors {left: parent.left; right: parent.right;}
        Item{
            anchors {left: parent.left;right: parent.right;}
            height: mealExpand.collapsedHeight
            ListItem.Standard {
                id: itemTitle
                text: name
                onClicked: {
                    mealExpand.expanded = !mealExpand.expanded
                    if (!mealExpand.expanded) {
                        mealExpand.clicked();
                        mealExpand.save(nameField.text, weight.text, carbs.text);
                    }
                    else mealExpand.clicked();
                }
                onPressAndHold: mealExpand.pressAndHold();
            }
        }
        Grid{
            anchors.horizontalCenter: parent.horizontalCenter
            spacing: units.gu(1)
            columns: 2
            horizontalItemAlignment: Grid.AlignRight
            verticalItemAlignment: Grid.AlignVCenter
            Label{
                text: "Name"
                font.pixelSize: FontUtils.sizeToPixels("large")
            }
            TextField {
                id: nameField
                text: (entry) ? entry.name : ""
                width: units.gu(17)
                font.pixelSize: FontUtils.sizeToPixels("large")
                placeholderText: "-"
                font.bold: true
                hasClearButton: false
                style: TextFieldStyle {
                    background: null
                }
                onTextChanged: itemTitle.text = text;
            }
            Label{
                text: "Weight"
                font.pixelSize: FontUtils.sizeToPixels("large")
            }
            TextField {
                id: weight
                width: units.gu(17)
                font.pixelSize: FontUtils.sizeToPixels("large")
                placeholderText: "-"
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                horizontalAlignment: TextInput.AlignLeft
                font.bold: true
                text: (entry.weight !== 0) ? entry.weight : ""
                hasClearButton: false
                style: TextFieldStyle {
                    background: null
                }
                secondaryItem: Label{
                    text: "g"
                    font.pixelSize: FontUtils.sizeToPixels("large")
                }
                validator: DoubleValidator {
                    bottom: 0
                    top: 1000
                    decimals: 0
                    notation: DoubleValidator.StandardNotation
                }
            }
            Label{
                text: "Carbs"
                font.pixelSize: FontUtils.sizeToPixels("large")
            }
            TextField {
                id: carbs
                width: units.gu(17)
                font.pixelSize: FontUtils.sizeToPixels("large")
                placeholderText: "-"
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                horizontalAlignment: TextInput.AlignLeft
                font.bold: true
                text: (entry.carbs !== 0) ? entry.carbs : ""
                hasClearButton: false
                style: TextFieldStyle {
                    background: null
                }
                secondaryItem: Label{
                    text: settings.carbsUnit
                    font.pixelSize: FontUtils.sizeToPixels("large")
                }
                validator: DoubleValidator {
                    bottom: 0
                    top: 999
                    decimals: 1
                    notation: DoubleValidator.StandardNotation
                }
            }
        }
    }
}

