#include <QDebug>
#include "carbsstore.h"
#include "meal.h"

#include <libintl.h>
#include "../constants.h"
#include "../Database/databaseworker.h"

CarbsStore::CarbsStore(QObject *parent)
    : QAbstractListModel(parent)
    , m_SqlTableCarbsStore(new QSqlQueryModel())
{
   readSqlData();
}
CarbsStore::~CarbsStore(){
    m_SqlTableCarbsStore->clear();
}
void CarbsStore::readSqlData(){
    m_SqlTableCarbsStore = DatabaseWorker::instance()->getAllMeals();
    while (m_SqlTableCarbsStore->canFetchMore()){
        m_SqlTableCarbsStore->fetchMore();
    }
    for (int rowIdx = 0; rowIdx < m_SqlTableCarbsStore->rowCount(); rowIdx++){
        Meal *newMeal = new Meal(this);
        newMeal->setId(m_SqlTableCarbsStore->data(m_SqlTableCarbsStore->index(rowIdx,IDX_MEAL_ID)).toInt());
        newMeal->setName(m_SqlTableCarbsStore->data(m_SqlTableCarbsStore->index(rowIdx,IDX_MEAL_NAME)).toString());
        newMeal->setWeight(m_SqlTableCarbsStore->data(m_SqlTableCarbsStore->index(rowIdx,IDX_MEAL_WEIGHT)).toDouble());
        newMeal->setCarbs(m_SqlTableCarbsStore->data(m_SqlTableCarbsStore->index(rowIdx,IDX_MEAL_CARBS)).toDouble());
        newMeal->setExpanded(false);
        beginInsertRows(QModelIndex(),m_meals.count(),m_meals.count());
        m_meals.append(newMeal);
        endInsertRows();
    }
    Q_EMIT countChanged();
}

int CarbsStore::rowCount(const QModelIndex &parent) const{
    Q_UNUSED(parent)
    return m_meals.count();
}
void CarbsStore::createEntry(){
    Meal *newMeal = new Meal(this);

    newMeal->setName("");
    newMeal->setWeight(0);
    newMeal->setCarbs(0);

    int newId = DatabaseWorker::instance()->saveMealEntry(newMeal);
    if(newId != -1){
        newMeal->setId(newId);
        beginInsertRows(QModelIndex(),m_meals.count(),m_meals.count());
        m_meals.append(newMeal);
        endInsertRows();
        Q_EMIT countChanged();
    }
}



void CarbsStore::saveEntry(const int &id,const QString &name, const double &weight, const double &carbs){
    int idx = indexOfId(id);
    Meal *meal = m_meals.value(idx);

    meal->setName(name);
    meal->setWeight(weight);
    meal->setCarbs(carbs);
    meal->setExpanded(false);

    int res = DatabaseWorker::instance()->updateMealEntry(meal);
    if (res != -1){
        emit dataChanged(index(m_meals.indexOf(meal)), index(m_meals.indexOf(meal)));
    }
}
QVariant CarbsStore::data(const QModelIndex &index, int role) const{
    switch(role){
    case RoleId:
        return m_meals.at(index.row())->id();
    case RoleName:
        return m_meals.at(index.row())->name();
    case RoleWeight:
        return m_meals.at(index.row())->weight();
    case RoleCarbs:
        return m_meals.at(index.row())->carbs();
    case RoleExpanded:
        return m_meals.at(index.row())->expanded();
    }
    return QVariant();
}
void CarbsStore::removeEntry(int id) {
   if (DatabaseWorker::instance()->deleteMealEntry(id)){
       int idx = indexOfId(id);
       emit beginRemoveRows(QModelIndex(),idx,idx);
       m_meals.takeAt(idx)->deleteLater();
       emit endRemoveRows();
       Q_EMIT countChanged();
   }
}
QHash<int, QByteArray> CarbsStore::roleNames() const{
    QHash<int, QByteArray> roles;
    roles.insert(RoleId,"id");
    roles.insert(RoleName,"name");
    roles.insert(RoleWeight,"weight");
    roles.insert(RoleCarbs,"carbs");
    roles.insert(RoleExpanded,"expanded");
    return roles;
}

int CarbsStore::indexOfId(const int &id){
    for (int i = 0; i < m_meals.size(); i++) {
        if (id == m_meals[i]->id())
            return i;
    }
    return -1;
}
