#ifndef CARBSSTORE_H
#define CARBSSTORE_H

#include <QObject>
#include <QSqlDatabase>
#include <QAbstractListModel>
#include <QSqlQueryModel>

class Meal;

class CarbsStore : public QAbstractListModel
{
    Q_OBJECT
    Q_PROPERTY(int count READ rowCount NOTIFY countChanged)

public:
    enum Roles{
        RoleId,
        RoleName,
        RoleWeight,
        RoleCarbs,
        RoleExpanded
    };

    explicit CarbsStore(QObject *parent = 0);
    ~CarbsStore();
    Q_INVOKABLE void createEntry();
    Q_INVOKABLE void saveEntry(const int &id, const QString &name, const double &weight, const double &carbs);
    Q_INVOKABLE void removeEntry(int id);

    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    QHash<int, QByteArray> roleNames() const;

protected:

signals:
    void countChanged();

private:
    QList<Meal*> m_meals;
    QSqlQueryModel* m_SqlTableCarbsStore;

    void readSqlData();
    int indexOfId(const int &id);
};

#endif // CARBSSTORE_H
